# $Id: CDDB.pm,v 1.17 1999/01/10 05:36:50 jzawodn Exp $

# $Revision: 1.17 $

package Net::CDDB;
require Exporter;
use IO::Socket;

@ISA = qw(Exporter);
@EXPORT = qw($cddb_server $cddb_port $cddb_localuser $cddb_localhost &cddb_query &cddb_read &cddb_inexact &cddb_lscat &cddb_sites);

my $revision = '$Revision: 1.17 $';
   $revision =~ m/(\d+\.\d+)/;
   $revision = $1;
   $VERSION  = $revision;
my $name     = "Net::CDDB";
my $full_name= "$name $VERSION";
my $socket;
   $debug    = 0;

$|++;

$cddb_server    = "cddb.cddb.com";
$cddb_port      = "888";
$cddb_localuser = "nobody";
$cddb_localhost = "localhost.localdomain";

#####################################################################3

sub _write_line {
    my $line = shift @_;
    print "$line\n" if $debug;
    print $socket "$line\r\n";
}

sub _read_line {

    my $line = <$socket>;
    print $line if $debug;
    $line =~ s/\r//o;
    chomp $line;
    return $line;
}

sub _read_lines {

    my $line;
    my @lines;

    $line = <$socket>;
    while ($line !~ /^\./o) {
        print $line if $debug;
        $line =~ s/\r//o;
        chomp $line;
        push @lines, $line;
        $line = <$socket>;
    }
    return @lines;
}

sub _connect {

    $socket = IO::Socket::INET->new(PeerAddr => $cddb_server,
                                    PeerPort => $cddb_port,
                                    Proto    => "tcp",
                                    Type     => SOCK_STREAM) or die "$!";

    _read_line;

} # end sub

sub _disconnect {

    _write_line("quit");
    close($socket);

} # end sub

sub _cddb_hello {
    my $msg = "cddb hello $cddb_localuser $cddb_localhost $full_name";
    _write_line($msg);
    _read_line;

}

sub _extract_stuff {
    my @lines = @_;
    my @returns;
    my $line;

    foreach $line (@lines) {
        if ($line =~ /^DTITLE/) {
            $line =~ /.*?=(.*)$/o;
            my ($a,$t) = _artist_title($1);
            push @returns, $a;
            push @returns, $t;
            print "TITLE: $1\n" if $debug;
        } elsif ($line =~ /^TTITLE/) {
            $line =~ /.*?=(.*)$/o;
            push @returns, $1;
            print "TITLE: $1\n" if $debug;
        }
    }
    return @returns;
}

sub _artist_title {
    my $line = shift @_;
    if ($line =~ m|(.*)\s+\/\s+(.*)\s*|g) {
        $a = $1;
        $t = $2;
    } else {
        $a = $line;
        $t = $line;
    }
    return ($a,$t);
}

sub cddb_query {
    my $disc_data = shift @_;

    _connect;
    _cddb_hello;

    _write_line("cddb query $disc_data");
    my $line = _read_line;

    $line =~ /^(\d+)\s+(\S+)\s+(\S+)\s+(.+)/o;
    my $status = $1;
    my $cat    = $2;
    my $discid = $3;
    my $dtitle = $4;

    my $artist;
    my $title;

    if ($status eq "200") {
        # Exact Match
        $cat = $2;
        ($artist,$title) = _artist_title($dtitle);
    } elsif ($status eq "211") {
        # Inexact Match
        $cat = "inexact";
        my @options = _read_lines;
    } elsif ($status eq "202") {
        # Not Found
    } elsif ($status eq "403") {
        # Entry Corrupt
    } elsif ($status eq "409") {
        # No Handshake
    }

    _disconnect;

    return($cat,$discid,$artist,$title);

} # end sub

sub cddb_sites {
    _connect;
    _cddb_hello;

    _write_line("sites");
    my $line = _read_line;
    my @lines;

    $line =~ /^(\d+)\s+(\S+)\s+(\S+)/o;
    my $status = $1;

    if ($status eq "210") {
        # Exact Match
        @lines = _read_lines;
    } elsif ($status eq "401") {
        # No Handshake
        @lines = ();
    }

    _disconnect;

    return(@lines);
}

sub cddb_inexact {
    my $disc_data = shift @_;

    _connect;
    _cddb_hello;

    _write_line("cddb query $disc_data");
    my $line = _read_line;
    return unless $line =~ /^211/o;

    my @lines = _read_lines;

    my %discs;

    foreach $line (@lines) {
        # format is: [cat] [discid] [dtitle]
        $line =~ /^(\S+)\s+(\S+)\s+(.+)$/o;
        my $ca = $1;
        my $id = $2;
        my $dt = $3;

        $discs{$id} = "[$ca] - $dt";
    }

    return %discs;
}

sub cddb_read {
    my $disc_id = shift @_;

    _connect;
    _cddb_hello;

    _write_line("cddb read $disc_id");
    my @lines = _read_lines;

    _disconnect;

    return _extract_stuff(@lines);

} # end sub

sub cddb_lscat {
    _connect;
    _cddb_hello;

    _write_line("cddb lscat");
    _read_line;
    my @lines = _read_lines;

    _disconnect;

    return @lines;
}

1;

__END__

=head1 NAME

Net::CDDB - Communicate with CDDB Servers using CDDB Protocol

=head1 SYNOPSIS

  use Net::CDDB;

  my $disc_data = `discid`;
  chomp($disc_data);

  my($cat,$disc_id,$artist,$title) = cddb_query($disc_data);

  print "Category: $cat\n";
  print "DiscID  : $disc_id\n";
  print "Artist  : $artist\n";
  print "Title   : $title\n";

  my($title,@tracks) = cddb_read("$cat $disc_id");
  my $num_tracks = $#tracks + 1;

  print "Album: $title\n";
  print "$num_tracks tracks\n";
  foreach (@tracks) {
      $num++;
      print "$num. $_\n";
  }

=head1 DESCRIPTION

Net:CDDB provides an easy to use functional (non-OO) interface for
communicating with Internet CDDB servers using the CDDB protocol (not
CDDB over HTTP).

See the L<"CDDB INFO"> section, below, for more info about CDDB
itself.

=head1 VARIABLES

Net::CDDB exports a minimal number of variables into the user's
namespace by default. They are all prefixed with `cddb' so as to
prevent obvious naming clases. They all correspond to information that
is required for a CDDB client to proivde to a CDDB server. They are:

=over 4

=item $cddb_server

The fully qualified domain name (or IP address) of the CDDB server
that you'd like to use. This will default to C<cddb.cddb.com> if you
don't specify one. Folks in other parts of the world will certainly
want to change this to something closer.

=item $cddb_port

The port number on which the CDDB server is listening. This will
default to 888 if you don't specify one. Since 888 is the CDDB
standard port, it is unlikely that you'll need to change this unless
you're talking to an experimental server.

=item $cddb_localuser

The username that you'd like to use to identify yourself with when
C<Net::CDDB> contacts the server. Though this will default to
C<nobody>, I hightly recommend that you supply a valid username.

=item $cddb_localhost

The hostname you'd like to use when identifying yourself with when
C<Net::CDDB> contacts the server. Most CDDB srevers combine
C<$cddb_localuser> and C<$cddb_localhost> to come up with your e-mail
address, so you should probably just use those values. This will
default to C<localhost.localdomain> if you don't specify.

=item $Net::CDDB::debug

Not exported by default, you may set C<$Net::CDDB::debug> to a
non-zero value if you'd like to enable debugging output. This would be
helpful if you're trying to track down a problem.

=back

=head1 METHODS

Net:CDDB exports a minimal number of methods into the user's namespace
by default. Most of them correspond to CDDB protocol verbs. They are:

=over 4

=item cddb_query($discinfo)

Performs a CDDB query on the given C<$discinfo>. In this case,
C<$discinfo> is a string which contains:

  <discid> <track offsets> <disc length in seconds>

For example, the C<$discinfo> string for Deep Forrest's I<Boheme> CD
looks like this: (line wrapped to improve readability)

  940a070c 12 150 8285 32097 51042 71992 86235 100345 105935 120932
  139472 158810 171795 2567

For more information about calcualting that information, see the
L<"TOOLS"> section late in this document.

C<cddb_query()> returns the following sort of array:

  ($category,$discid,$artist,$title)

If the query is successful and an exact match is found, all of those
variables will contain data. You will probably want to hang on to
C<$category> and C<$discid> so that you can pass them to
C<cddb_read()> and get all the info you're really after.

If the query is successful but only finds inexact matches,
C<$category> will contain the word C<inexact> and all the other values
will be undef. You will probably want to call C<cddb_inexact()> and
pass it the same C<$discdata> that you passed this function.

If the query fails for some reason (not found, corrupt database entry,
or improper protocol handshake), all the values will be undef. There
is currently no way to find out which of those three errors occurred.

=item cddb_read($category,$discid)

Performs a CDDB read command on a given category and discid. One would
usually call

=item cddb_inexact($discdata)

Query on C<$discdata> and retrieve the list of inexact matches so that
you (or the user) can select the proper one. The C<$discdata> argument
is exactly the same as the one used in C<cddb_query()>.

C<cddb_inexact()> returns a hash that looks like this:

  $discs{$discid} = "[$category] - $dtitle";

where C<$dtitle> is the DTITLE record on the CDDB database.

I<This will eventually change to a list structure. I just haven't
finished that part of the code yet.>

=item cddb_lscat()

Retrieve a list of the music categories the server
uses. C<cddb_lscat()> requires no arguments and simply returns a
list. It can't get much easier than that.

=item cddb_sites()

Retrieve a list of the sites the CDDB server knows
about. C<cddb_sites()> requires no arguments and simply returns a
list. It really doesn't get any easier.

=back

=head1 CDDB INFO

CDDB is an acronym for ``CD Database''. It also refers to the CDDB
protocol (TCP port 888) which is used to query CDDB databases to
retrieve and update information about music CDs.

For information about CDDB and the CDDB protocol spec, please visit
http://www.cddb.com/ which is the official CDDB Web site.

This module implements most of the CDDB protocol version 1.

=head1 TOOLS

There are many tools for calculating the disc info needed to query the
CDDB. Some of the more popular CD ``rippers'' (such as cdda2wav) can
extract the needed informaton. However, not being happy with having to
use a whole different software package to do the job, I wrote a small
C program that does exactly what I need and no more. As a result, the
program is quite small.

So, check your favorite CD ripper. If it doesn't do the job, grab my
B<discid> program and see if it does the trick for you. I will
eventually re-write it in Perl and include it in this distribution,
but untill then you can grab it from the following address:

  http://www.wcnet.org/~jzawodn/c/discid/

=head1 BUGS

We could have better error reporting.

When an album has multiple discids in the CDDB database,
C<cddb_read()> will return them as a comma separated list. We should
handle that properly.

C<cddb_inexact()> really needs to use arrary references in the hash it
returns instead of the hackish approach we use now.

This module is ``read-only'' which is to say that it doesn't allow you
to submit CDDB records to a server. I don't currently have a need for
that functionality, so I haven't added it.

=head1 AUTHOR AND COPYRIGHT

Copyright 1998, Jeremy D. Zawodny <jzawodn@wcnet.org>

Net::CDDB is covered under the GNU Public License (GPL) version 2 or
later.

The Net::CDDB Web site is located at:

  http://www.wcnet.org/~jzawodn/perl/Net-CDDB/

=cut
